// Claim to "override" Ext so that this code is loaded early
Ext.define("joo_runtime_init", {
    override: "Ext"
});
Ext.Class.registerPreprocessor('__accessors__', function(Class, data) {
    if (data.__accessors__) {
        if (data.__accessors__.statics) {
            Object.defineProperties(Class, data.__accessors__.statics);
            delete data.__accessors__.statics;
        }
        Object.defineProperties(Class.prototype, data.__accessors__);
        delete data.__accessors__;
    }
});
Ext.ClassManager.registerPostprocessor('__factory__', function(className, cls, data) {
    if (data.__factory__) {
        var value = data.__factory__();
        this.set(className, value);
        this.triggerCreated(className);
        return false;
    }
    return true;
});
(function() {
    var wrapConstructor = function(Class) {
            return function() {
                // console.log("*** called constructor of " + Ext.getClassName(Class) + " for the first time.");
                Class.__doInit__();
                return Class.prototype.constructor.apply(this, arguments);
            };
        };
    var wrapStaticMember = function(staticMemberName) {
            return {
                get: function() {
                    // console.log("*** read " + Ext.getClassName(this) + "." + staticMemberName + " for the first time.");
                    this.__doInit__();
                    return this[staticMemberName];
                },
                set: function(value) {
                    // console.log("*** set " + Ext.getClassName(this) + "." + staticMemberName + " for the first time.");
                    this.__doInit__();
                    this[staticMemberName] = value;
                },
                enumerable: true,
                configurable: true
            };
        };
    Ext.Class.registerPreprocessor('statics', function(Class, data) {
        var statics = data.statics;
        if (statics) {
            var initStatics = statics.__initStatics__;
            if (!initStatics) {
                Class.addStatics(statics);
            } else {
                delete statics.__initStatics__;
                var originalConstructor = data.hasOwnProperty("constructor") ? data.constructor : null;
                data.constructor = wrapConstructor(Class);
                for (var staticMemberName in statics) {
                    Object.defineProperty(Class, staticMemberName, wrapStaticMember(staticMemberName));
                }
                Class.__doInit__ = function() {
                    delete this.__doInit__;
                    // self-destruct!
                    // remove all initializing interceptors:
                    if (originalConstructor) {
                        this.prototype.constructor = originalConstructor;
                        data.constructor = originalConstructor;
                    } else {
                        delete this.prototype.constructor;
                        delete data.constructor;
                    }
                    for (var staticMemberName in statics) {
                        delete this[staticMemberName];
                    }
                    // now, define the real statics:
                    this.addStatics(statics);
                    initStatics();
                };
            }
            delete data.statics;
        }
    });
})();

// Claim to "override" Ext.namespace so that this code is loaded / included right after Ext.namespace has been defined!
Ext.define("AS3_override", {
    override: "Ext.namespace",
    requires: [
        "joo_runtime_init"
    ]
});
Class = {
    $isClass: true,
    $className: "Class",
    __isInstance__: function(object) {
        return typeof object === "function" && !!object.$isClass;
    }
};
Vector$object = Array;
Ext.apply(Ext.namespace("AS3"), {
    // function assert(cond : Object, file : String, line : uint, column : uint) : void
    assert: function(cond, file, line, column) {
        if (!cond)  {
            throw new Error(file + "(" + line + ":" + column + "): assertion failed");
        }
        
    },
    bind: function bind(object, boundMethodName) {
        var method = object[boundMethodName];
        if (object.hasOwnProperty(boundMethodName)) {
            return method;
        }
        var boundMethod = method.bind(object);
        Object.defineProperty(object, boundMethodName, {
            writable: true,
            // TODO: for backwards-compatibility only
            value: boundMethod
        });
        return boundMethod;
    },
    is: function(object, type) {
        if (!type || object === undefined || object === null) {
            return false;
        }
        if (Ext.isFunction(type.__isInstance__)) {
            return type.__isInstance__(object);
        }
        if (!Ext.isFunction(type)) {
            return false;
        }
        // constructor or instanceof may return false negatives:
        if (object.constructor === type || object instanceof type) {
            return true;
        }
        // special case for special observables, e.g. classes:
        if (object.isObservable && (type.$className === "Ext.mixin.Observable" || type.$className === "Ext.util.Observable")) {
            return true;
        }
        if (typeof object === 'object') {
            var objectType;
            if (object.isInstance) {
                objectType = object.self;
            } else {
                var typeName = object.xclass || object.xtype && Ext.ClassManager.getNameByAlias("widget." + object.xtype);
                if (typeName) {
                    objectType = Ext.ClassManager.get(typeName);
                }
            }
            if (AS3.isAssignableFrom(type, objectType)) {
                return true;
            }
        }
        return false;
    },
    isAssignableFrom: function(type, clazz) {
        if (!type || !clazz) {
            return false;
        }
        if (type === clazz) {
            return true;
        }
        if (clazz.prototype instanceof type) {
            return true;
        }
        if (type.$className) {
            var mixins = clazz.prototype.mixins;
            if (Ext.isObject(mixins)) {
                // fast path for jangaroo compiler: no explicit check if the class really is the expected class
                if (mixins[type.$className]) {
                    return true;
                }
                // as mixinIds might not be unique, have an additional type check if mixin is found
                if (type.prototype.mixinId && AS3.is(mixins[type.prototype.mixinId], type)) {
                    return true;
                }
                // fallback to checking the type of every entry in mixin mapping to make sure the mixin is not registered
                // under a custom id (e.g. Ext.panel.Panel has an entry mixins: { docking: "Ext.container.DockingContainer"; } )
                for (var key in mixins) {
                    var mixin = mixins[key];
                    if (mixin && AS3.isAssignableFrom(type, mixin.self)) {
                        return true;
                    }
                }
            }
        }
        return false;
    },
    as: function(object, type) {
        return AS3.is(object, type) ? object : null;
    },
    cast: function(type, value) {
        if (value === undefined || value === null) {
            return value;
        }
        var typePrototype = type.prototype;
        if (type.$className && typePrototype && typeof value === "object" && !value.isInstance && !value.xclass && !value.xtype) {
            if (typePrototype.hasOwnProperty("xtype") && Ext.isString(typePrototype.xtype)) {
                value.xtype = typePrototype.xtype;
            } else {
                var alias;
                var aliasType;
                if (typePrototype.hasOwnProperty("type") && Ext.isString(typePrototype.type)) {
                    aliasType = typePrototype.type;
                } else if (typePrototype.hasOwnProperty("alias") && Ext.isArray(typePrototype.alias) && Ext.isString(alias = typePrototype.alias[0])) {
                    aliasType = alias.substr(alias.indexOf(".") + 1);
                }
                if (aliasType) {
                    value.type = aliasType;
                }
            }
            value.xclass = type.$className;
        } else if (!AS3.is(value, type)) {
            throw new TypeError("Value cannot be cast to " + Ext.getClassName(type) + ": " + value);
        }
        return value;
    },
    getBindable: function(object, property) {
        if (object.isInstance) {
            this._checkConfig(object, property, "get");
            return object.getConfig(property);
        } else {
            return object[property];
        }
    },
    setBindable: function(object, property, value) {
        if (object.isInstance) {
            this._checkConfig(object, property, "set");
            object.setConfig(property, value);
        } else {
            object[property] = value;
        }
        return value;
    },
    _checkConfig: function(object, property, accessPrefix) {
        if (!Ext.Config.map[property]) {
            var accessorName = accessPrefix + Ext.String.capitalize(property);
            this.trace("[WARN]", "AS3." + accessPrefix + "Bindable() called on " + Ext.getClassName(object) + "#" + property + ", which is not registered as a config. Still using method " + accessorName + "().");
            //this is a workaround: Ext seems not to manage some config properties properly. E.g. Button#menu
            //we do this by our own
            Ext.Config.get(property);
        }
    },
    addEventListener: function(eventDispatcher, flexEventClass, flexEventConstantName, flexEventListener, extEventOptions) {
        var flexEventName = flexEventClass[flexEventConstantName];
        var extEventName = flexEventName.substr(2).toLowerCase();
        var extEventListener = function() {
                return flexEventListener(new flexEventClass(flexEventName, arguments));
            };
        if (eventDispatcher.isInstance) {
            var args = Array.prototype.slice.call(arguments, 4);
            args.splice(0, 0, extEventName, extEventListener);
            eventDispatcher.addListener.apply(eventDispatcher, args);
        } else {
            if (!eventDispatcher.listeners) {
                eventDispatcher.listeners = {};
            }
            eventDispatcher.listeners[extEventName] = extEventOptions ? Ext.apply({
                fn: extEventListener
            }, extEventOptions) : extEventListener;
        }
    }
});

// hack: provide empty objects for E4X classes for browsers not supporting E4X (all but Firefox)
// so that at least the runtime does not try to load them:
if (typeof XML === "undefined") {
    XML = {};
}
if (typeof XMLList === "undefined") {
    XMLList = {};
}

// built-in as well as Ext Error constructor called as function unfortunately always create a new Error object,
// so we have to create a class that uses the original Error prototype, but also works as an Ext class:
Ext.require("AS3_override");
Ext.define("AS3.Error", {
    __factory__: function() {
        var AS3Error = function(message, /*String*/
            id) /*:int*/
            {
                this.message = message || "";
                this.id = id || 0;
            };
        AS3Error.$isClass = true;
        AS3Error.$className = "AS3.Error";
        AS3Error.__isInstance__ = function(object) {
            // use built-in Error class for instanceof check:
            return object instanceof Error;
        };
        AS3Error.superclass = Error.prototype;
        AS3Error.prototype = Object.create(Error.prototype);
        AS3Error.prototype.self = AS3Error;
        AS3Error.prototype.constructor = AS3Error;
        AS3Error.prototype.callParent = Ext.Base.prototype.callParent;
        return AS3Error;
    }
});

Ext.require("AS3_override");
Ext.define("AS3.int_", {
    __factory__: function() {
        var int_ = function(num) {
                return num >> 0;
            };
        int_.MAX_VALUE = 2147483647;
        int_.MIN_VALUE = -2.147483648E9;
        int_.$className = "AS3.int_";
        int_.__isInstance__ = function(object) {
            return (object instanceof Number || typeof object === 'number') && object >> 0 === object + 0;
        };
        // "+ 0" converts Number to number!
        return int_;
    }
});

// function trace(...msg) : void
Ext.require("AS3_override");
Ext.define("AS3.trace", {
    __factory__: function() {
        var console, defaultLogLevel;
        var lookup = function(consoleObject, traceFunctionName) {
                if (consoleObject && consoleObject[traceFunctionName]) {
                    console = consoleObject;
                    defaultLogLevel = traceFunctionName;
                    return true;
                }
                return false;
            };
        try {
            lookup(window.console, "log") || lookup(window.runtime, "trace") || lookup(window, "trace") || lookup(window.opera, "postError");
        } catch (e) {}
        // ignore
        var LOG_LEVEL_PATTERN = /^\[(LOG|DEBUG|TRACE|INFO|WARN|ERROR)]\s*(.*)$/;
        var trace = !console ? Ext.emptyFn : function() {
                // don't use Array.prototype.map, as it is not available in all browsers and has not yet been polyfilled:
                var params = [];
                for (var i = 0; i < arguments.length; i++) {
                    params.push(String(arguments[i]));
                }
                var msg = params.join(" ");
                var logLevel = defaultLogLevel;
                var logLevelMatches = msg.match(LOG_LEVEL_PATTERN);
                if (logLevelMatches) {
                    var specialLogLevel = logLevelMatches[1].toLowerCase();
                    try {
                        if (console[specialLogLevel]) {
                            // special log level supported by this console:
                            logLevel = specialLogLevel;
                            msg = logLevelMatches[2];
                        }
                    } catch (e) {}
                }
                // ignore
                console[logLevel]("AS3: " + msg);
            };
        return trace;
    }
});

Ext.require("AS3_override");
Ext.define("AS3.uint_", {
    __factory__: function() {
        var uint_ = function(num) {
                return num >>> 0;
            };
        uint_.MAX_VALUE = 4.294967295E9;
        uint_.MIN_VALUE = 0;
        uint_.$className = "AS3.uint_";
        uint_.__isInstance__ = function(object) {
            return (object instanceof Number || typeof object === 'number') && object >>> 0 === object + 0;
        };
        // "+ 0" converts Number to number!
        return uint_;
    }
});

Ext.define("ArgumentError", function(ArgumentError) {
    /*package {

/**
 * The ArgumentError class represents an error that occurs when the arguments supplied in a function do not match the arguments defined for that function. This error occurs, for example, when a function is called with the wrong number of arguments, an argument of the incorrect type, or an invalid argument.
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/./ArgumentError.html#includeExamplesSummary">View the examples</a></p>
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7ecf.html Responding to error events and status
 *
 * /
public dynamic class ArgumentError extends Error {
  /**
   * Creates an ArgumentError object.
   * @param message A string associated with the error.
   *
   * /
  public*/
    function ArgumentError$(message) /*:String = ""*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        AS3.Error.prototype.constructor.call(this, message);
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "AS3.Error",
        constructor: ArgumentError$,
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("joo.JavaScriptObject", function(JavaScriptObject) {
    /*package joo {

/**
 * This is a marker class indicating that any subclass is intended to be used as a typed JSON object, i.e.
 * no enumerable "meta" properties may be defined by the Jangaroo Runtime.
 * /
public class JavaScriptObject {

  /**
   * Create a typed JavaScript object from the given JSON object.
   * @param config the JSON object to copy into this JavaScriptObject. If no JSON object is given, this JavaScriptObject
   *   is initially empty.
   * /
  public*/
    function JavaScriptObject$(config) /*:Object = null*/
    {
        if (arguments.length <= 0)  {
            config = null;
        }
        
        for (var key in /*:String*/
        config) {
            this[key] = config[key];
        }
    }
    /*

}
}

============================================== Jangaroo part ==============================================*/
    return {
        constructor: JavaScriptObject$
    };
});

(function(theGlobalObject) {
    // define alias "js" for the top-level package, so that name-clashes in AS3 can be resolved:
    theGlobalObject.js = theGlobalObject;
    // define Object.getPrototypeOf() if not already defined:
    if (!Object.getPrototypeOf) {
        Object.getPrototypeOf = function getPrototypeOf(object) {
            if (!object || typeof object !== "object") {
                throw new TypeError();
            }
            return object.__proto__ || object.constructor['superclass'] || object.constructor.prototype;
        };
    }
    // this is the best we can do...
    // defined here to avoid global name space pollution and unnecessary closures:
    function clone(object) {
        var empty = function() {};
        empty.prototype = object;
        return new empty();
    }
    function copyFromTo(source, target) {
        for (var m in source) {
            target[m] = source[m];
        }
    }
    function createGetQualified(create) {
        return (function(name) {
            var object = theGlobalObject;
            if (name) {
                var parts = name.split(".");
                for (var i = 0; i < parts.length; ++i) {
                    var subobject = object[parts[i]];
                    try {
                        if (String(subobject).indexOf("[JavaPackage") == 0) {
                            subobject = null;
                        }
                    } catch (e) {}
                    // ignore
                    if (!subobject) {
                        if (create) {
                            subobject = object[parts[i]] = {};
                        } else {
                            return null;
                        }
                    }
                    object = subobject;
                }
            }
            return object;
        });
    }
    if (!theGlobalObject.joo) {
        theGlobalObject.joo = {};
    }
    joo.getOrCreatePackage = createGetQualified(true);
    joo.getQualifiedObject = createGetQualified(false);
    // add event listener using Ext JS config object style:
    joo.addEventListener = function addEventListener(config, eventName, eventType, callback) {
        if (!config.listeners) {
            config.listeners = {};
        }
        config.listeners[eventName] = function() {
            callback(new eventType(arguments));
        };
    };
    // built-in Error constructor called as function unfortunately always creates a new Error object, so we have to emulate it:
    joo.Error = function(message, /*String*/
    id) /*:int*/
    {
        this.message = message || "";
        this.id = id || 0;
    };
    joo.Error.prototype = Error.prototype;
    joo.JavaScriptObject = function(config) {
        if (config) {
            for (var m in config) {
                this[m] = config[m];
            }
        }
    };
    joo.is = function(object, type) {
        if (!type || object === undefined || object === null) {
            return false;
        }
        // special case meta-class Class:
        if (type === Class) {
            return !!object["$class"];
        }
        // constructor or instanceof may return false negatives:
        if (object instanceof type || object.constructor === type) {
            return true;
        }
        // special case int and uint:
        if (type === $$int || type === $$uint) {
            if (object instanceof Number || typeof object === 'number') {
                // thanks http://stackoverflow.com/questions/3885817/how-to-check-if-a-number-is-float-or-integer
                return (type === $$uint ? object >>> 0 : object >> 0) === object + 0;
            }
        }
        // "+ 0" converts Number to number!
        else if (typeof object === 'object') {
            // only Objects may implement a Jangaroo interface:
            var typeDeclaration = type["$class"];
            if (typeDeclaration) {
                return typeDeclaration.isInstance(object);
            }
        }
        return false;
    };
    joo.as = function(object, type) {
        return joo.is(object, type) ? object : null;
    };
    joo.boundMethod = function boundMethod(object, methodName) {
        return object['$$b_' + methodName] || (typeof object[methodName] === "function" ? (object['$$b_' + methodName] = function() {
            return object[methodName].apply(object, arguments);
        }) : object[methodName]);
    };
    joo.initField = function(self, fieldName) {
        self[fieldName] = self[fieldName]();
    };
    /*
    unsupported ActionScript features during bootstrap:
      - private non-static members (use internal instead)
      - field initializers (initialize in constructor instead)
      - dynamic class loading + resource bundles
      - implicit empty constructor (must have explicit constructor)
      - Array methods like forEach (not supported by all browsers)
      - all classes must reside within the joo package

     Caveat: static code blocks are executed immediately

     */
    joo.classLoader = {
        prepare: function(packageDef, classDef, inheritanceLevel, memberFactory, publicStaticMethodNames, dependencies, runtimeApiVersion, compilerVersion) {
            joo.runtimeApiVersion = runtimeApiVersion;
            joo.compilerVersion = compilerVersion;
            var classMatch = classDef.match(/^\s*((public|internal|final|dynamic)\s+)*class\s+([A-Za-z][a-zA-Z$_0-9]*)(\s+extends\s+([a-zA-Z$_0-9.]+))?(\s+implements\s+([a-zA-Z$_0-9.,\s]+))?\s*$/);
            var className = classMatch[3];
            var $extends = classMatch[5];
            var constructor = {};
            // also used for collecting static member
            var superConstructor = $extends ? joo.getQualifiedObject($extends) : Object;
            constructor.superclass = superConstructor.prototype;
            // Ext Core compatibility
            var prototype = clone(superConstructor.prototype);
            prototype["super$" + inheritanceLevel] = superConstructor;
            var privateStatics = {};
            var members = memberFactory(privateStatics);
            var staticInitializers = [];
            for (var i = 0; i < members.length; ++i) {
                var memberDeclaration = members[i];
                switch (typeof memberDeclaration) {
                    case "function":
                        staticInitializers.push(memberDeclaration);
                        break;
                    case "string":
                        var isStatic = memberDeclaration.match(/\bstatic\b/);
                        var isPrivate = memberDeclaration.match(/\bprivate\b/);
                        var target = isStatic ? isPrivate ? privateStatics : constructor : prototype;
                        var member = members[++i];
                        if (typeof member == "function") {
                            var methodName = memberDeclaration.match(/function\s+([a-zA-Z$_0-9]+)/)[1];
                            if (methodName == className) {
                                // found constructor!
                                // add collected public static members to the real constructor
                                copyFromTo(constructor, member);
                                joo[className] = constructor = member;
                                constructor.prototype = prototype;
                            } else {
                                target[methodName] = member;
                            }
                        } else {
                            copyFromTo(member, target);
                        };
                }
            }
            Object.defineProperty(prototype, "constructor", {
                value: constructor,
                writable: true,
                configurable: true
            });
            for (var s = 0; s < staticInitializers.length; s++) {
                staticInitializers[s]();
            }
        },
        init: function() {}
    };
})(// ignore
this);

Ext.define("DefinitionError", function(DefinitionError) {
    /*package {

/**
 * The DefinitionError class represents an error that occurs when user code attempts to define an identifier that is already defined. This error commonly occurs in redefining classes, interfaces, and functions.
 * /
public dynamic class DefinitionError extends Error {
  /**
   * Creates a new DefinitionError object.
   * @param message A string associated with the error.
   * /
  public*/
    function DefinitionError$(message) /*:String = ""*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        AS3.Error.prototype.constructor.call(this, message);
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "AS3.Error",
        constructor: DefinitionError$,
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("SecurityError", function(SecurityError) {
    /*package {

/**
 * The <code>SecurityError</code> exception is thrown when some type of security violation takes place.
 * <p>Examples of security errors:</p>
 * <ul>
 * <li>An unauthorized property access or method call is made across a security sandbox boundary.</li>
 * <li>An attempt was made to access a URL not permitted by the security sandbox.</li>
 * <li>A socket connection was attempted to an unauthorized port number, e.g. a port above 65535.</li>
 * <li>An attempt was made to access the user's camera or microphone, and the request to access the device was denied by the user.</li></ul>
 * <p><a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/./SecurityError.html#includeExamplesSummary">View the examples</a></p>
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7ecf.html Responding to error events and status
 *
 * /
public dynamic class SecurityError extends Error {
  /**
   * Creates a new SecurityError object.
   * @param message A string associated with the error.
   * /
  public*/
    function SecurityError$(message) /*:String = ""*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        AS3.Error.prototype.constructor.call(this, message);
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "AS3.Error",
        constructor: SecurityError$,
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("VerifyError", function(VerifyError) {
    /*package {


/**
 * The VerifyError class represents an error that occurs when a malformed or corrupted SWF file is encountered.
 * @see flash.display.Loader
 * @see http://help.adobe.com/en_US/as3/dev/WS5b3ccc516d4fbf351e63e3d118a9b90204-7ecf.html Responding to error events and status
 *
 * /
public dynamic class VerifyError extends Error {
  /**
   * Creates a new VerifyError object.
   * @param message Contains the message associated with the VerifyError object.
   *
   * /
  public*/
    function VerifyError$(message) /*:String = ""*/
    {
        if (arguments.length <= 0)  {
            message = "";
        }
        
        AS3.Error.prototype.constructor.call(this, message);
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "AS3.Error",
        constructor: VerifyError$,
        requires: [
            "AS3.Error"
        ]
    };
});

Ext.define("joo.SystemClassLoader", function(SystemClassLoader) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {

public class SystemClassLoader {

  public static const classDeclarationsByName : Object/*<String,JooClassDeclaration>* / =*/
    function classDeclarationsByName$static_() {
        SystemClassLoader.classDeclarationsByName = ({});
    }
    /*;

  public var debug : Boolean;

  public*/
    function SystemClassLoader$() {
        this.debug = joo.debug;
    }
    /*

  public*/
    function prepare() /*...params*/
    /* : JooClassDeclaration*/
    {
        var params = Array.prototype.slice.call(arguments);
        var packageDef = /* : String*/
            params[0];
        var metadata = /* : Object*/
            {};
        for (var i = /*:int*/
            1; typeof params[i] == "object"; i++) {
            SystemClassLoader.addToMetadata(metadata, params[i]);
        }
        var classDef = /* : String*/
            params[i++];
        var inheritanceLevel = /*:**/
            params[i++];
        if (typeof inheritanceLevel !== "number") {
            // No inheritance level? Runtime version < 0.8.0.
            // Be backwards-compatible, so that we at least generate the right error:
            i--;
        }
        var memberFactory = /* : Function*/
            params[i++];
        var publicStaticMethodNames = /* : Array*/
            params[i++];
        var dependencies = /* : Array*/
            params[i++];
        var runtimeApiVersion = /*:String*/
            params[i++];
        var compilerVersion = /*:String*/
            params[i++];
        var cd = /* : JooClassDeclaration*/
            this.createClassDeclaration(packageDef, metadata, classDef, inheritanceLevel, memberFactory, publicStaticMethodNames, dependencies);
        if (!SystemClassLoader.isRuntimeCompatible(runtimeApiVersion, compilerVersion)) {
            throw new AS3.Error("Runtime version " + joo.runtimeApiVersion + "/" + joo.compilerVersion + " and class version " + runtimeApiVersion + "/" + compilerVersion + " of " + cd.fullClassName + " do not match. " + "Please recompile with the correct compiler version or replace jangaroo-runtime[-debug].js.");
        }
        SystemClassLoader.classDeclarationsByName[cd.fullClassName] = cd;
        return cd;
    }
    /*

  private static*/
    function toVersionParts$static(version) /*:String*/
    /*:Array*/
    /*int*/
    {
        var parts = /*:Array*/
            (version || "").split(/\.|-preview/);
        for (var i = /*:uint*/
            0; i < parts.length; ++i) {
            parts[i] = parseInt(parts[i], 10);
            if (parts[i] < 0) {
                parts[i] = AS3.int_.MIN_VALUE - parts[i];
            }
        }
        return parts;
    }
    /*

  /**
   * @internal
   * Used for runtime-to-class compatibility check. <code>internal</code>, not <code>private</code>, for testing purposes.
   * A class runtimeApiVersion/compilerVersion is compatible with the current Runtime if
   * <ul>
   * <li>the runtimeApiVersion matches the Runtime's runtimeApiVersion exactly and</li>
   * <li>the compilerVersion is lower or the same as the Runtime's compilerVersion.</li>
   * </ul>
   * @param runtimeApiVersion the runtime API version to check for compatibility with the loaded Jangaroo Runtime
   * @param compilerVersion the compiler version to check for compatibility with the loaded Jangaroo Runtime
   * @return Boolean whether the given runtimeApiVersion/compilerVersion is compatible with the current Runtime.
   * /
  internal static*/
    function isRuntimeCompatible$static(runtimeApiVersion, /*:String*/
    compilerVersion) /*:String*/
    /*:Boolean*/
    {
        // The Runtime API version must match exactly:
        if (runtimeApiVersion !== joo.runtimeApiVersion) {
            return false;
        }
        // The compiler versions must be compared part-by-part:
        var runtimeCompilerVersionParts = /*:Array*/
            /*.<uint>*/
            toVersionParts$static(joo.compilerVersion);
        var compilerVersionParts = /*:Array*/
            /*.<uint>*/
            toVersionParts$static(compilerVersion);
        for (var i = /*:uint*/
            0; i < runtimeCompilerVersionParts.length; ++i) {
            if (compilerVersionParts[i] !== runtimeCompilerVersionParts[i]) {
                // The given class must be compiled with an older or the same compiler version as the Runtime:
                return compilerVersionParts[i] < runtimeCompilerVersionParts[i];
            }
        }
        return true;
    }
    /*

  internal static*/
    function addToMetadata$static(metadata, /*:Object*/
    annotation) /*:**/
    /*:void*/
    {
        for (var m in /*:String*/
        annotation) {
            metadata[m] = annotation[m];
        }
    }
    /*

  protected*/
    function createClassDeclaration(packageDef, /*:String*/
    metadata, /*:Object*/
    classDef, /*:String*/
    inheritanceLevel, /*:int*/
    memberFactory, /*:Function*/
    publicStaticMethodNames, /*:Array*/
    dependencies) /*:Array*/
    /* : JooClassDeclaration*/
    {
        return AS3.cast(joo.JooClassDeclaration, new joo.JooClassDeclaration(packageDef, metadata, classDef, inheritanceLevel, memberFactory, publicStaticMethodNames, dependencies).init());
    }
    /*

  public*/
    function getClassDeclaration(fullClassName) /* : String*/
    /* : NativeClassDeclaration*/
    {
        var cd = /* : NativeClassDeclaration*/
            SystemClassLoader.classDeclarationsByName[fullClassName];
        if (!cd) {
            var constructor_ = /* : Function*/
                joo.getQualifiedObject(fullClassName);
            if (constructor_) {
                // create JooClassDeclaration for native classes:
                cd = this.createNativeClassDeclaration(fullClassName, constructor_).init();
                SystemClassLoader.classDeclarationsByName[fullClassName] = cd;
            }
        }
        return cd;
    }
    /*

  /**
   * @param className
   * @return NativeClassDeclaration the class declaration with the given name.
   * @throws Error - ClassNotFound
   * /
  public*/
    function getRequiredClassDeclaration(className) /* : String*/
    /* : NativeClassDeclaration*/
    {
        var cd = /* : NativeClassDeclaration*/
            this.getClassDeclaration(className);
        if (!cd) {
            throw new AS3.Error("Class not found: " + className);
        }
        return cd;
    }
    /*

  protected*/
    function createNativeClassDeclaration(fullClassName, /* : String*/
    nativeClass) /* : Function*/
    /* : NativeClassDeclaration*/
    {
        return new joo.NativeClassDeclaration().create(fullClassName, nativeClass);
    }
    /*

  public*/
    function init() /*... classes*/
    /* :Function*/
    {
        var classes = Array.prototype.slice.call(arguments);
        return null;
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        debug: false,
        constructor: SystemClassLoader$,
        prepare: prepare,
        createClassDeclaration: createClassDeclaration,
        getClassDeclaration: getClassDeclaration,
        getRequiredClassDeclaration: getRequiredClassDeclaration,
        createNativeClassDeclaration: createNativeClassDeclaration,
        init: init,
        statics: {
            classDeclarationsByName: undefined,
            isRuntimeCompatible: isRuntimeCompatible$static,
            addToMetadata: addToMetadata$static,
            __initStatics__: function() {
                classDeclarationsByName$static_();
            }
        },
        uses: [
            "AS3.Error",
            "AS3.int_",
            "joo.JooClassDeclaration",
            "joo.NativeClassDeclaration"
        ]
    };
});

Ext.define("joo.StandardClassLoader", function(StandardClassLoader) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {

public class StandardClassLoader extends SystemClassLoader {

  private static*/
    var classDeclarations$static;
    /* : Array*/
    /* =*/
    function classDeclarations$static_() {
        classDeclarations$static = ([]);
    }
    
    /*

  private var imports : Array;

  public*/
    function StandardClassLoader$() {
        joo.SystemClassLoader.prototype.constructor.apply(this, arguments);
        this.imports$2 = [];
    }
    /*

  override protected*/
    function createClassDeclaration(packageDef, /*:String*/
    metadata, /*:Object*/
    classDef, /*:String*/
    inheritanceLevel, /*:int*/
    memberFactory, /*:Function*/
    publicStaticMethodNames, /*:Array*/
    dependencies) /*:Array*/
    /*:JooClassDeclaration*/
    {
        var cd = /* : JooClassDeclaration*/
            new joo.JooClassDeclaration(packageDef, metadata, classDef, inheritanceLevel, memberFactory, publicStaticMethodNames, dependencies);
        classDeclarations$static.push(cd);
        // remember all created classes for later initialization.
        return cd;
    }
    /*

  /**
   * Import the class given by its fully qualified class name (package plus name).
   * All imports are collected in a hash and can be used in the #complete() callback function.
   * @param fullClassName : String the fully qualified class name (package plus name) of the class to load and import.
   * /
  public*/
    function import_(fullClassName) /* : String*/
    /* : void*/
    {
        this.imports$2.push(fullClassName);
    }
    /*

  /**
   * Run the static main method of a class given by its fully qualified name (package plus name), handing through the
   * given arguments (args).
   * The main method is executed after all classes are completed (see #complete()).
   * @param mainClassName : String the fully qualified name (package plus name) or the constructor function
   *   of the class to run.
   * @param args the arguments to hand over to the main method of the given class.
   * /
  public*/
    function run(mainClassName) /* : String, ...args*/
    /* : void*/
    {
        var this$ = this;
        var args = Array.prototype.slice.call(arguments, 1);
        this.complete(function() /* : void*/
        {
            var mainClass = /* : NativeClassDeclaration*/
                this$.getRequiredClassDeclaration(mainClassName).init();
            mainClass.constructor_["main"].apply(null, args);
        });
    }
    /*

  /**
   * Explicitly initialize the static members of the given class (constructor function).
   * If the class is not yet initialized, Initializers of static variables and static code blocks are called (once).
   * This is only necessary when you want to access constants of a class without importing the class,
   * or when you have loaded the class explicitly and want its static code to execute.
   * Explicit initializing is <i>not</i> necessary when you
   * - import the class or
   * - load the class and use the constructor or a static method of the class. This will trigger initialization
   *   automatically.
   * @param classes the classes (type Function) to initialize.
   * @return Function the last initialized class (constructor function). It only makes sense to use the return value
   *   if you use this method with exactly one parameter.
   * /
  public override*/
    function init() /*... classes*/
    /* : Function*/
    {
        var classes = Array.prototype.slice.call(arguments);
        var clazz;
        /* : Function*/
        for (var i = /*:int*/
            0; i < classes.length; ++i) {
            clazz = classes[i];
            if (clazz && clazz["$class"]) {
                clazz = AS3.cast(joo.NativeClassDeclaration, clazz["$class"]).init().constructor_;
            }
        }
        return clazz;
    }
    /*

  /**
   * Tell Jangaroo to load and initialize all required classes, then call the given function.
   * The function receives an import hash, which can be used in pure JavaScript in a 'with' statement
   * (Jangaroo does not support 'with', there, you would use import declarations!) like this:
   * <pre>
   * joo.classLoader.import_("com.custom.Foo");
   * joo.classLoader.complete(function(imports){with(imports){
   *   Foo.doSomething("bar");
   * }});
   * </pre>
   * @param onCompleteCallback : Function
   * @return void
   * /
  public*/
    function complete(onCompleteCallback) /* : Function = undefined*/
    /* : void*/
    {
        this.initNativeClasses();
        if (onCompleteCallback) {
            this.doCompleteCallbacks([
                onCompleteCallback
            ]);
        }
    }
    /*

  protected*/
    function initNativeClasses() /* : void*/
    {
        for (var i = /*:int*/
            0; i < classDeclarations$static.length; i++) {
            var classDeclaration = /*:JooClassDeclaration*/
                classDeclarations$static[i];
            // init native class patches immediately:
            if (classDeclaration.isNative()) {
                classDeclaration.init();
            }
        }
    }
    /*

  protected*/
    function doCompleteCallbacks(onCompleteCallbacks) /* : Array*/
    /*Function*/
    /* : void*/
    {
        if (onCompleteCallbacks.length) {
            var importMap = /* : Object*/
                {};
            for (var j = /*:int*/
                0; j < this.imports$2.length; j++) {
                var fullClassName = /*:String*/
                    this.imports$2[j];
                var className = /* : String*/
                    fullClassName.substring(fullClassName.lastIndexOf(".") + 1);
                importMap[className] = joo.classLoader.getRequiredClassDeclaration(fullClassName).init().constructor_;
            }
            for (var i = /*:int*/
                0; i < onCompleteCallbacks.length; ++i) {
                onCompleteCallbacks[i](importMap);
            }
        }
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "joo.SystemClassLoader",
        imports$2: null,
        constructor: StandardClassLoader$,
        createClassDeclaration: createClassDeclaration,
        import_: import_,
        run: run,
        init: init,
        complete: complete,
        initNativeClasses: initNativeClasses,
        doCompleteCallbacks: doCompleteCallbacks,
        statics: {
            classDeclarations: undefined,
            __initStatics__: function() {
                classDeclarations$static_();
            }
        },
        requires: [
            "joo.SystemClassLoader"
        ],
        uses: [
            "joo.JooClassDeclaration",
            "joo.NativeClassDeclaration"
        ]
    };
});

Ext.define("joo.DynamicClassLoader", function(DynamicClassLoader) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {
public class DynamicClassLoader extends StandardClassLoader {

  private static*/
    function isEmpty$static(object) /* : Object*/
    /* : Boolean*/
    {
        //noinspection LoopStatementThatDoesntLoopJS
        for (var m in /*:String*/
        object) {
            return false;
        }
        return true;
    }
    /*

  public static var INSTANCE:DynamicClassLoader;

  private var resourceByPath : Object =*/
    function resourceByPath_() {
        this.resourceByPath$3 = ({});
    }
    /*;
  private var onCompleteCallbacks : Array/*<Function>* / =*/
    function onCompleteCallbacks_() {
        this.onCompleteCallbacks$3 = ([]);
    }
    /*;

  public*/
    function DynamicClassLoader$() {
        joo.StandardClassLoader.prototype.constructor.apply(this, arguments);
        resourceByPath_.call(this);
        onCompleteCallbacks_.call(this);
        pendingDependencies_.call(this);
        pendingClassState_.call(this);
        joo.classLoader = DynamicClassLoader.INSTANCE = this;
    }
    /*

  /**
   * Keep record of all classes whose dependencies still have to be loaded.
   * /
  private var pendingDependencies : Array/*<JooClassDeclaration>* / =*/
    function pendingDependencies_() {
        this.pendingDependencies$3 = ([]);
    }
    /*;
  /**
   * false => pending
   * true => loading
   * /
  private var pendingClassState : Object/*<String,Boolean>* / =*/
    function pendingClassState_() {
        this.pendingClassState$3 = ({});
    }
    /*;

  override public*/
    function prepare() /*...params*/
    /*:JooClassDeclaration*/
    {
        var params = Array.prototype.slice.call(arguments);
        var cd = /*:JooClassDeclaration*/
            AS3.cast(joo.JooClassDeclaration, joo.StandardClassLoader.prototype.prepare.apply(this, params));
        this.pendingDependencies$3.push(cd);
        this.fireDependency(cd.fullClassName);
        return cd;
    }
    /*

  public*/
    function addDependency(dependency) /*:String*/
    /*:void*/
    {
        this.pendingClassState$3[dependency] = true;
    }
    /*

  public*/
    function fireDependency(dependency) /*:String*/
    /*:void*/
    {
        if (delete this.pendingClassState$3[dependency]) {
            //      if (this.debug) {
            //        trace("prepared class " + dependency + ", removed from pending classes.");
            //      }
            if (this.onCompleteCallbacks$3.length) {
                this.loadPendingDependencies$3();
                if (isEmpty$static(this.pendingClassState$3)) {
                    this.doCompleteCallbacks(this.onCompleteCallbacks$3);
                }
            }
        }
    }
    /*

  override protected*/
    function doCompleteCallbacks(onCompleteCallbacks) /* : Array*/
    /*Function*/
    /*:void*/
    {
        var this$ = this;
        this.onCompleteCallbacks$3 = [];
        // "invoke later":
        joo.getQualifiedObject("setTimeout")(function() /* : void*/
        {
            this$.initNativeClasses();
            this$.internalDoCompleteCallbacks$3(onCompleteCallbacks);
        }, 0);
    }
    /*

  private*/
    function internalDoCompleteCallbacks(onCompleteCallbacks) /* : Array*/
    /*Function*/
    /*:void*/
    {
        joo.StandardClassLoader.prototype.doCompleteCallbacks.call(this, onCompleteCallbacks);
    }
    /*

  // separate factory function to move the anonymous function out of the caller's scope:
  private*/
    function createClassLoadErrorHandler(fullClassName, /*:String*/
    url) /*:String*/
    /*:Function*/
    {
        var this$ = this;
        return function() /*:void*/
        {
            this$.classLoadErrorHandler(fullClassName, url);
        };
    }
    /*

  public*/
    function classLoadErrorHandler(fullClassName, /*:String*/
    url) /*:String*/
    /*:void*/
    {
        AS3.trace("[ERROR]", "Jangaroo Runtime: Class " + fullClassName + " not found at URL [" + url + "].");
    }
    /*

  /**
   * Import the class given by its fully qualified class name (package plus name).
   * All imports are collected in a hash and can be used in the #complete() callback function.
   * Additionally, the DynamicClassLoader tries to load the class from a URL if it is not present on #complete().
   * @param fullClassName : String the fully qualified class name (package plus name) of the class to load and import.
   * /
  public override*/
    function import_(fullClassName) /* : String*/
    /* : void*/
    {
        var Ext = /*:**/
            joo.getQualifiedObject("Ext");
        Ext.require(fullClassName);
    }
    /*

  override public*/
    function run(mainClassName) /* : String, ...args*/
    /*:void*/
    {
        var args = Array.prototype.slice.call(arguments, 1);
        this.load$3(mainClassName);
        args.splice(0, 0, mainClassName);
        joo.StandardClassLoader.prototype.run.apply(this, args);
    }
    /*

  private*/
    function load(fullClassName) /* : String*/
    /* : void*/
    {
        var resourcePathMatch = /*:Array*/
            fullClassName.match(/^resource:(.*)$/);
        if (resourcePathMatch) {
            this.loadResource$3(resourcePathMatch[1]);
            return;
        }
        if (!this.getClassDeclaration(fullClassName)) {
            if (this.onCompleteCallbacks$3.length == 0) {
                if (this.pendingClassState$3[fullClassName] === undefined) {
                    // we are not yet in completion phase: just add to pending classes:
                    this.pendingClassState$3[fullClassName] = false;
                }
            } else //          if (this.debug) {
            //            trace("added to pending classes: "+fullClassName+".");
            //          }
            {
                if (this.pendingClassState$3[fullClassName] !== true) {
                    // trigger loading:
                    this.pendingClassState$3[fullClassName] = true;
                    var url = /*:String*/
                        joo.getRelativeClassUrl(fullClassName);
                    //          if (this.debug) {
                    //            trace("triggering to load class " + fullClassName + " from URL " + url + ".");
                    //          }
                    var script = /*:Object*/
                        joo.loadScriptAsync(url);
                    // script.onerror does not work in IE, but since this feature is for debugging only, we don't mind:
                    script.onerror = this.createClassLoadErrorHandler$3(fullClassName, script['src']);
                }
            }
        }
    }
    /*

  private static const*/
    var RESOURCE_TYPE_STRING$static = /*:String*/
        "String";
    /*
  private static const*/
    var RESOURCE_TYPE_IMAGE$static = /*:String*/
        "Image";
    /*
  private static const*/
    var RESOURCE_TYPE_AUDIO$static = /*:String*/
        "Audio";
    /*
  private static const*/
    var RESOURCE_TYPE_BY_EXTENSION$static;
    /*:Object*/
    /* =*/
    function RESOURCE_TYPE_BY_EXTENSION$static_() {
        RESOURCE_TYPE_BY_EXTENSION$static = ({
            "txt": RESOURCE_TYPE_STRING$static,
            "csv": RESOURCE_TYPE_STRING$static,
            "png": RESOURCE_TYPE_IMAGE$static,
            "gif": RESOURCE_TYPE_IMAGE$static,
            "jpg": RESOURCE_TYPE_IMAGE$static,
            "jpeg": RESOURCE_TYPE_IMAGE$static,
            "mp3": RESOURCE_TYPE_AUDIO$static,
            "ogg": RESOURCE_TYPE_AUDIO$static,
            "wav": RESOURCE_TYPE_AUDIO$static
        });
    }
    
    /*
  // TODO: map more extensions, also for video etc.
  // TODO: improvement: instead of extensions, we could do a HEAD request to the path and map the Content-Type to media/resource type.

  private*/
    function loadResource(path) /*:String*/
    /*:void*/
    {
        var this$ = this;
        var resource = /*:Object*/
            this.resourceByPath$3[path];
        if (!resource) {
            var dotPos = /*:int*/
                path.lastIndexOf('.');
            var extension = /*:String*/
                path.substring(dotPos + 1);
            var resourceType = /*:String*/
                RESOURCE_TYPE_BY_EXTENSION$static[extension];
            if (resourceType) {
                if (resourceType === RESOURCE_TYPE_STRING$static) {
                    var xhr = /*:Object*/
                        new (joo.getQualifiedObject('XMLHttpRequest'))();
                    xhr.open('GET', joo.resolveUrl("joo/classes/" + path));
                    xhr.onreadystatechange = function() /*:void*/
                    {
                        if (xhr.readyState === 4) {
                            delete xhr.onreadystatechange;
                            // only fire once!
                            this$.resourceByPath$3[path] = xhr.responseText;
                            this$.fireDependency("resource:" + path);
                        }
                    };
                    xhr.send(null);
                    return;
                }
                var resourceTypeClass = /*:Class*/
                    joo.getQualifiedObject(resourceType);
                if (resourceTypeClass) {
                    this.resourceByPath$3[path] = resource = new (resourceTypeClass)();
                    if (resourceType === RESOURCE_TYPE_IMAGE$static) {
                        this.addDependency("resource:" + path);
                        resource.onload = function() /*:void*/
                        {
                            this$.fireDependency("resource:" + path);
                        };
                        resource.onerror = function(m) /*:**/
                        /*:void*/
                        {
                            AS3.trace("[WARN]", "Error while loading resource " + path + ": " + m);
                            // however, we do not want dynamic loading to fail completely:
                            this$.fireDependency("resource:" + path);
                        };
                    } else if (resourceType === RESOURCE_TYPE_AUDIO$static) {
                        if (!resource['canPlayType']("audio/" + extension)) {
                            // try another MIME type / extension:
                            var fallbackExtension = /*:String*/
                                findFallback$static(resource);
                            if (!fallbackExtension) {
                                return;
                            }
                            path = path.substring(0, dotPos) + "." + fallbackExtension;
                        }
                        resource.preload = "auto";
                    }
                    // Embed -> load early, but don't wait for load like with images.
                    resource.src = joo.resolveUrl("joo/classes/" + path);
                } else {
                    AS3.trace("[WARN]", "Resource type " + resourceType + " not supported by client, ignoring resource " + path);
                }
            } else {
                AS3.trace("[WARN]", "Ignoring unsupported media type of file " + path);
            }
        }
    }
    /*

  private static const*/
    var AUDIO_FALLBACK_ORDER$static;
    /*:Array*/
    /* =*/
    function AUDIO_FALLBACK_ORDER$static_() {
        AUDIO_FALLBACK_ORDER$static = ([
            "mp3",
            "ogg",
            "wav"
        ]);
    }
    
    /*
  private static*/
    var AUDIO_FALLBACK_EXTENSION$static = /*:String*/
        null;
    /*
  private static*/
    function findFallback$static(audio) /*:Object*/
    /*:String*/
    {
        if (AUDIO_FALLBACK_EXTENSION$static === null) {
            for (var i = /*:int*/
                0; i < AUDIO_FALLBACK_ORDER$static.length; i++) {
                var fallback = /*:String*/
                    AUDIO_FALLBACK_ORDER$static[i];
                if (audio['canPlayType']("audio/" + fallback)) {
                    return AUDIO_FALLBACK_EXTENSION$static = fallback;
                }
            }
            AS3.trace("[WARN]", "Could not find any audio extension that this client can play (" + AUDIO_FALLBACK_ORDER$static.join(",") + "), no sound available.");
            AUDIO_FALLBACK_EXTENSION$static = "";
        }
        return AUDIO_FALLBACK_EXTENSION$static;
    }
    /*

  public*/
    function getResource(path) /*:String*/
    /*:Object*/
    {
        return this.resourceByPath$3[path];
    }
    /*

  /**
   * Tell Jangaroo to load and initialize all required classes, then call the given function.
   * The function receives an import hash, which can be used in pure JavaScript in a 'with' statement
   * (Jangaroo does not support 'with', there, you would use import declarations!) like this:
   * <pre>
   * joo.classLoader.import_("com.custom.Foo");
   * joo.classLoader.complete(function(imports){with(imports){
   *   Foo.doSomething("bar");
   * }});
   * </pre>
   * @param onCompleteCallback : Function
   * @return void
   * /
  public override*/
    function complete(onCompleteCallback) /* : Function = undefined*/
    /* : void*/
    {
        var Ext = /*:**/
            joo.getQualifiedObject("Ext");
        Ext.onReady(onCompleteCallback);
    }
    /*

  private static*/
    function defaultOnCompleteCallback$static() /* : void*/
    {
        AS3.trace("[INFO]", "Jangaroo Runtime: All classes loaded!");
    }
    /*

  private*/
    function loadPendingDependencies() /*:void*/
    {
        for (var j = /*:int*/
            0; j < this.pendingDependencies$3.length; ++j) {
            var dependencies = /* : Array*/
                AS3.cast(joo.JooClassDeclaration, this.pendingDependencies$3[j]).getDependencies();
            for (var i = /*:int*/
                0; i < dependencies.length; ++i) {
                this.load$3(dependencies[i]);
            }
        }
        this.pendingDependencies$3 = [];
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "joo.StandardClassLoader",
        constructor: DynamicClassLoader$,
        prepare: prepare,
        addDependency: addDependency,
        fireDependency: fireDependency,
        doCompleteCallbacks: doCompleteCallbacks,
        internalDoCompleteCallbacks$3: internalDoCompleteCallbacks,
        createClassLoadErrorHandler$3: createClassLoadErrorHandler,
        classLoadErrorHandler: classLoadErrorHandler,
        import_: import_,
        run: run,
        load$3: load,
        loadResource$3: loadResource,
        getResource: getResource,
        complete: complete,
        loadPendingDependencies$3: loadPendingDependencies,
        statics: {
            INSTANCE: null,
            RESOURCE_TYPE_BY_EXTENSION: undefined,
            AUDIO_FALLBACK_ORDER: undefined,
            __initStatics__: function() {
                RESOURCE_TYPE_BY_EXTENSION$static_();
                AUDIO_FALLBACK_ORDER$static_();
            }
        },
        requires: [
            "joo.StandardClassLoader",
            "joo.SystemClassLoader"
        ],
        uses: [
            "AS3.trace",
            "joo.JooClassDeclaration"
        ]
    };
});

Ext.define("joo.ILocaleSupport", function(ILocaleSupport) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 * /
package joo {

/**
 * This interface provides the foundation of localization support with ExtJS.
 * Its singleton instance <code>joo.localeSupport</code> maintain a current locale and loads a locale-specific subclass of any
 * class ending with <code>_properties</code>.
 * <p>The locale is saved in and retrieved from a Cookie whose name, path, and domain
 * can be configured.</p>
 * <p>In your main HTML file, load <code>joo/before-ext-load.js</code>, before loading Ext and the Jangaroo Runtime.
 * Activate locale-specific resource bundle overrides loading as follows:</p>
 * <pre>
 * new joo.ILocaleSupport(["en", "de"]);
 * </pre>
 * <p>You can create resource bundle classes by creating <code>*_<i>locale</i>.properties</code>
 * files below the <code>src/main/joo</code> directory. These files are translated to JavaScript
 * code by the Jangaroo properties compiler (Maven goal: <code>properties</code>).
 * Each resource bundle must consist of properties files for each supported locale, where
 * the default locale suffix is suppressed.
 * For example for supported locales <code>["en", "de"]</code> (which makes <code>"en"</code>
 * the default locale), you need properties files <code>My.properties</code> and <code>My_de.properties</code>.</p>
 * <p>To change the locale, your application code must call</p>
 * <pre>
 * joo.localeSupport.setLocale(newLocale);
 * </pre>
 *
 * @see #ILocaleSupport
 * @see #getLocale
 * @see #setLocale
 *
 * @author Manuel Ohlendorf
 * @author Frank Wienberg
 * /
public interface ILocaleSupport {

  /**
   * Set before any resource bundle is loaded. Default is <code>"joo.locale"</code>
   * @see #getLocale
   * /
  function get localeCookieDomain():String;

  //noinspection JSUnusedGlobalSymbols
  function getSupportedLocales():Array;

  function getDefaultLocale():String;

  function readLocaleFromCookie():String;

  /**
   * Sets the current locale to the given locale or, if the given locale is not supported,
   * to the longest match in the supported locales, or the default locale if there is no match.
   * @param newLocale the locale to use for resource bundle class loading
   * @return String the supported locale that has actually been set
   * @see #ILocaleSupport
   * /
  function setLocale(newLocale:String):String;

  /**
   * Find the supported locale with the longest match against the given locale
   * /
  function findSupportedLocale(locale:String):String;

  /**
   * Return the locale currently used for resource bundle loading.
   * On the first call of this method, the locale is read from the Cookie given by
   * <code>localeCookieName</code> (default "joo.locale"), <code>localeCookiePath</code>
   * (<code>window.location.pathname</code>) and <code>localeCookieDomain</code>
   * (<code>null</code>).
   * If there is no such Cookie, the browser's <code>navigator</code> object is asked for its
   * language. If the locale still is not determined, the <code>defaultLocale</code> is used.
   * This value if stored using <code>setLocale()</code>.
   *
   * @return the locale currently used for resource bundle loading.
   *
   * @see #ILocaleSupport
   * @see #setLocale
   * /
  function getLocale():String;

}
}

============================================== Jangaroo part ==============================================*/
    return {};
});

Ext.define("joo.NativeClassDeclaration", function(NativeClassDeclaration) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {
public class NativeClassDeclaration {

  internal static const RESOURCE_BUNDLE_PATTERN:RegExp = /_properties$/;

  internal static*/
    function createEmptyConstructor$static(prototype_) /* : Object*/
    /* : Function*/
    {
        var emptyConstructor = /* : Function*/
            function() /* : void*/
            {};
        if (prototype_) {
            emptyConstructor.prototype = prototype_;
        }
        return emptyConstructor;
    }
    /*

  internal static const STATE_LOADED : int = 0;
  internal static const STATE_COMPLETING : int = 1;
  internal static const STATE_COMPLETED : int = 2;
  internal static const STATE_INITIALIZING : int = 3;
  internal static const STATE_MEMBERS_INITIALIZED : int = 4;
  internal static const STATE_INITIALIZED : int = 5;

  public var
          fullClassName : String,
          constructor_ : Function,
          publicConstructor : Function,
          state  : int =*/
    function state_() {
        this.state = (NativeClassDeclaration.STATE_LOADED);
    }
    /*,
          Public : Function,
          superClassDeclaration : NativeClassDeclaration,
          interfaces : Array;

  public*/
    function NativeClassDeclaration$() {
        state_.call(this);
    }
    /*

  public*/
    function create(fullClassName, /* : String*/
    publicConstructor) /* : Function*/
    /* : NativeClassDeclaration*/
    {
        this.fullClassName = fullClassName;
        this.publicConstructor = publicConstructor;
        if (publicConstructor) {
            try {
                this.publicConstructor["$class"] = this;
            } catch (e) /*:**/
            {}
        }
        // ignore that expando properties do not work with certain native objects in certain browsers, e.g. IE7 / XMLHttpRequest
        return this;
    }
    /*

  public*/
    function complete() /* : NativeClassDeclaration*/
    {
        if (this.state < NativeClassDeclaration.STATE_COMPLETING) {
            this.state = NativeClassDeclaration.STATE_COMPLETING;
            this.doComplete();
            this.state = NativeClassDeclaration.STATE_COMPLETED;
        }
        return this;
    }
    /*

  internal*/
    function doComplete() /* : void*/
    {
        this.interfaces = [];
        this.constructor_ = AS3.cast(Class, this.publicConstructor) === AS3.Error ? joo.getQualifiedObject("joo.Error") : this.publicConstructor;
        this.Public = NativeClassDeclaration.createEmptyConstructor(this.publicConstructor.prototype);
    }
    /*

  public*/
    function isInstance(obj) /*:Object*/
    /*:Boolean*/
    {
        return obj instanceof this.publicConstructor;
    }
    /*

  private static*/
    var initializationDepth$static = /*:String*/
        "";
    /*

  public*/
    function init() /* : NativeClassDeclaration*/
    {
        if (this.state < NativeClassDeclaration.STATE_INITIALIZING) {
            this.complete();
            this.state = NativeClassDeclaration.STATE_INITIALIZING;
            if (joo.classLoader.debug) {
                AS3.trace("[DEBUG]", "Jangaroo Runtime: initializing class " + initializationDepth$static + this.fullClassName);
                initializationDepth$static += "  ";
            }
            this.doInit();
            if (joo.classLoader.debug) {
                initializationDepth$static = initializationDepth$static.substr(0, initializationDepth$static.length - 2);
            }
            this.state = NativeClassDeclaration.STATE_INITIALIZED;
        } else if (this.state < NativeClassDeclaration.STATE_INITIALIZED && !this.fullClassName.match(NativeClassDeclaration.RESOURCE_BUNDLE_PATTERN)) {
            AS3.trace("[WARN]", "Jangaroo Runtime: cyclic static initializer dependency in " + this.fullClassName);
        }
        return this;
    }
    /*

  internal*/
    function doInit() /* : void*/
    {}
    /*

  public*/
    function getQualifiedName() /* : String*/
    {
        // AS uses namespace notation (::) to separate package and class name,
        // so replace the last dot ('.') by a double-colon ('::'):
        return this.fullClassName.replace(/\.([^.]+)$/, "::$1");
    }
    /*

  public*/
    function toString() /* : String*/
    {
        return this.fullClassName;
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        fullClassName: null,
        constructor_: null,
        publicConstructor: null,
        Public: null,
        superClassDeclaration: null,
        interfaces: null,
        constructor: NativeClassDeclaration$,
        create: create,
        complete: complete,
        doComplete: doComplete,
        isInstance: isInstance,
        init: init,
        doInit: doInit,
        getQualifiedName: getQualifiedName,
        toString: toString,
        statics: {
            RESOURCE_BUNDLE_PATTERN: /_properties$/,
            createEmptyConstructor: createEmptyConstructor$static,
            STATE_LOADED: 0,
            STATE_COMPLETING: 1,
            STATE_COMPLETED: 2,
            STATE_INITIALIZING: 3,
            STATE_MEMBERS_INITIALIZED: 4,
            STATE_INITIALIZED: 5
        },
        uses: [
            "AS3.Error",
            "AS3.trace"
        ]
    };
});

Ext.define("joo.JooClassDeclaration", function(JooClassDeclaration) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {

public class JooClassDeclaration extends NativeClassDeclaration {

  public static const STATE_EVENT_AFTER_INIT_MEMBERS:String = 'afterInitMembers';
  private static*/
    var STATE_BY_EVENT$static = /*:Object*/
        null;
    function static$0() {
        STATE_BY_EVENT$static = {
            'afterInitMembers': joo.NativeClassDeclaration.STATE_MEMBERS_INITIALIZED
        };
    }
    /*

  internal var
          package_ : Object,
          type : String =*/
    function type_() {
        this.type = (joo.MemberDeclaration.MEMBER_TYPE_CLASS);
    }
    /*,
          namespace_ : String =*/
    function namespace__() {
        this.namespace_ = (joo.MemberDeclaration.NAMESPACE_INTERNAL);
    }
    /*,
          className : String,
          native_ : Boolean = false,
          extends_ : String = "Object",
          level : int = -1,
          privateStatics : Object,
          memberDeclarations : * /* Function, then Array * /,
          memberDeclarationsByQualifiedName : Object,
          staticInitializers : Array/*<MemberDeclaration|Function>* /,
          publicStaticMethodNames : Array,
          implementingClasses: Array/*Class* /,
          dependencies : Array,
          stateListeners: Object;
  /**
   * The metadata (annotations) associated with this class.
   * /
  public var metadata : Object;

  private static const*/
    var DECLARATION_PATTERN_CLASS$static = /*:RegExp*/
        /^\s*((public|internal|final|dynamic)\s+)*class\s+([a-zA-Z$_0-9]+)(\s+extends\s+([a-zA-Z$_0-9.]+))?(\s+implements\s+([a-zA-Z$_0-9.,\s]+))?\s*$/;
    /*
  private static const*/
    var DECLARATION_PATTERN_INTERFACE$static = /*:RegExp*/
        /^\s*((public|internal)\s+)?interface\s+([a-zA-Z$_0-9]+)(\s+extends\s+([a-zA-Z$_0-9.,\s]+))?\s*$/;
    /*
  private static const*/
    var DECLARATION_PATTERN_OTHER$static = /*:RegExp*/
        /^\s*((public|internal)\s+)?(const|var|function|namespace)\s+([a-zA-Z$_0-9]+)\s*$/;
    /*

  public*/
    function JooClassDeclaration$(packageDef, /*:String*/
    metadata, /*:Object*/
    classDef, /*:String*/
    inheritanceLevel, /*:int*/
    memberDeclarations, /*:Function*/
    publicStaticMethodNames, /*:Array*/
    dependencies) /*:Array*/
    {
        joo.NativeClassDeclaration.prototype.constructor.call(this);
        namespace__.call(this);
        type_.call(this);
        this.stateListeners = {};
        this.metadata = metadata;
        var packageName = /* : String*/
            packageDef.split(/\s+/)[1] || "";
        this.package_ = joo.getOrCreatePackage(packageName);
        var classMatch = /* : Array*/
            classDef.match(DECLARATION_PATTERN_CLASS$static);
        var interfaces;
        /* : String*/
        if (classMatch) {
            this.className = classMatch[3];
            if (classMatch[5]) {
                this.extends_ = classMatch[5];
            }
            interfaces = classMatch[7];
        } else {
            classMatch = classDef.match(DECLARATION_PATTERN_INTERFACE$static);
            if (classMatch) {
                this.className = classMatch[3];
                this.type = joo.MemberDeclaration.MEMBER_TYPE_INTERFACE;
                interfaces = classMatch[5];
            } else {
                classMatch = classDef.match(DECLARATION_PATTERN_OTHER$static);
                if (classMatch) {
                    this.className = classMatch[4];
                    this.type = classMatch[3];
                }
            }
        }
        if (!classMatch) {
            throw new AS3.Error("SyntaxError: \"" + classDef + "\" does not match.");
        }
        this.level = inheritanceLevel;
        this.namespace_ = classMatch[2];
        var fullClassName = /* : String*/
            this.className;
        if (packageName) {
            fullClassName = packageName + "." + this.className;
        }
        this.interfaces = interfaces ? interfaces.split(/\s*,\s*/) : [];
        this.memberDeclarations = memberDeclarations;
        this.publicStaticMethodNames = publicStaticMethodNames;
        this.dependencies = dependencies;
        this.privateStatics = {};
        this.publicConstructor = joo.getQualifiedObject(fullClassName);
        if (this.publicConstructor) {
            this.native_ = true;
        } else if (this.isClass() || this.isInterface()) {
            this.package_[this.className] = this.publicConstructor = createInitializingConstructor$static(this);
            for (var i = /*:int*/
                0; i < publicStaticMethodNames.length; i++) {
                this.createInitializingStaticMethod(publicStaticMethodNames[i]);
            }
        } else if (this.isFunction()) {
            this.package_[this.className] = createInitializingPackageMethod$static(this);
        } else if (this.isConst() || this.isVar()) {
            this.package_[this.className] = typeof this.memberDeclarations === "function" ? createInitializingPackageField$static(this) : this.memberDeclarations;
        }
        this.create(fullClassName, this.publicConstructor);
        this._processMetadata();
    }
    // for early annotation processing like adding dependencies
    /*

  public*/
    function addStateListener(state, /*:String*/
    listener) /*:Function*/
    /*:void*/
    {
        if (this.state >= STATE_BY_EVENT$static[state]) {
            // when already past this state, call back immediately:
            listener(this);
        } else {
            var stateListeners = /*:Array*/
                this.stateListeners[state];
            if (!stateListeners) {
                this.stateListeners[state] = stateListeners = [];
            }
            stateListeners.push(listener);
        }
    }
    /*

  public*/
    function removeStateListener(state, /*:String*/
    listener) /*:Function*/
    /*:void*/
    {
        var stateListeners = /*:Array*/
            this.stateListeners[state];
        if (stateListeners) {
            var pos = /*:int*/
                stateListeners.indexOf(listener);
            if (pos !== -1) {
                stateListeners.splice(pos, 1);
            }
        }
    }
    /*

  public*/
    function isClass() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_CLASS;
    }
    /*

  public*/
    function isInterface() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_INTERFACE;
    }
    /*

  public*/
    function isFunction() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_FUNCTION;
    }
    /*

  public*/
    function isConst() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_CONST;
    }
    /*

  public*/
    function isVar() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_VAR;
    }
    /*

  internal*/
    function addToInterfaces(clazz) /*:Function*/
    /*:void*/
    {
        var scd = /*:JooClassDeclaration*/
            AS3.as(this.superClassDeclaration, JooClassDeclaration);
        if (scd) {
            scd.addToInterfaces(clazz);
        }
        for (var i = /*:int*/
            0; i < this.interfaces.length; i++) {
            AS3.cast(JooClassDeclaration, this.interfaces[i]).addImplementingClass(clazz);
        }
    }
    /*

  internal*/
    function addImplementingClass(clazz) /*:Function*/
    /*:void*/
    {
        //trace("#### adding " + clazz + " to interface " + fullClassName + ":");
        var implementingClasses = /*:Array*/
            [];
        //trace("####   before: " + this.implementingClasses.join(", "));
        for (var i = /*:int*/
            0; i < this.implementingClasses.length; i++) {
            var implementingClass = /*:Function*/
                this.implementingClasses[i];
            // do not add new clazz if it or a superclass is already in the set:
            if (clazz === implementingClass || clazz.prototype instanceof implementingClass) {
                //trace("####   " + implementingClass + " already present, nothing changed.");
                return;
            }
            // class or superclass already added!
            // remove all subclasses from the set (keep only non-subclasses):
            if (!(implementingClass.prototype instanceof clazz)) {
                implementingClasses.push(implementingClass);
            }
        }
        implementingClasses.push(clazz);
        this.implementingClasses = implementingClasses;
        //trace("####   after: " + this.implementingClasses.join(", "));
        this.addToInterfaces(clazz);
    }
    /*

  override public*/
    function isInstance(obj) /*:Object*/
    /*:Boolean*/
    {
        return this.Public ? this.isInterface() ? this.implementingClasses.some(function(implementingClass) /*:Function*/
        /*:Boolean*/
        {
            return obj instanceof implementingClass;
        }) : obj instanceof this.Public : // cannot invoke super, since BootstrapClassLoader does not support super calls!
        false;
    }
    // class not even completed, cannot have instances!
    /*

  public*/
    function isNamespace() /* : Boolean*/
    {
        return this.type === joo.MemberDeclaration.MEMBER_TYPE_NAMESPACE;
    }
    /*

  public*/
    function isNative() /* : Boolean*/
    {
        return this.native_;
    }
    /*

  internal override*/
    function doComplete() /* : void*/
    {
        this.superClassDeclaration = joo.classLoader.getRequiredClassDeclaration(this.extends_);
        this.superClassDeclaration.complete();
        var proto = /*:Object*/
            this.native_ ? this.publicConstructor.prototype : new (this.superClassDeclaration.Public)();
        this.Public = joo.NativeClassDeclaration.createEmptyConstructor(proto);
    }
    /*

  internal*/
    function initMembers() /* : void*/
    {
        this.staticInitializers = [];
        var memberDeclarations = /*:Array*/
            this.memberDeclarations(this.privateStatics);
        this.memberDeclarations = [];
        this.memberDeclarationsByQualifiedName = {};
        this.constructor_ = this.isNative() ? this.publicConstructor : null;
        var metadata = /*:Object*/
            {};
        for (var i = /*:int*/
            0; i < memberDeclarations.length; ++i) {
            var item = /*:**/
                memberDeclarations[i];
            switch (typeof item) {
                case "function":
                    this.staticInitializers.push(item);
                    break;
                case "string":
                    var memberDeclaration = /*:MemberDeclaration*/
                        joo.MemberDeclaration.create(item);
                    if (memberDeclaration) {
                        memberDeclaration.metadata = metadata;
                        metadata = {};
                        if (!memberDeclaration.isNative()) {
                            if (++i >= memberDeclarations.length) {
                                throw new AS3.Error(this + ": Member expected after modifiers '" + item + "'.");
                            }
                            var member = /*:**/
                                memberDeclarations[i];
                        }
                        switch (memberDeclaration.memberType) {
                            case joo.MemberDeclaration.MEMBER_TYPE_FUNCTION:
                                this.initMethod(memberDeclaration, AS3.cast(Function, member));
                                break;
                            case joo.MemberDeclaration.MEMBER_TYPE_CLASS:
                                //noinspection UnnecessaryLocalVariableJS
                                var helperInheritanceLevel = /*:int*/
                                    member;
                                var helperMemberDeclarations = /*:Function*/
                                    memberDeclarations[++i];
                                var helperStatics = /*:Array*/
                                    memberDeclarations[++i];
                                var secondaryClass = /*:NativeClassDeclaration*/
                                    joo.classLoader.prepare("package " + this.fullClassName, item, helperInheritanceLevel, helperMemberDeclarations, helperStatics, [], joo.runtimeApiVersion, joo.compilerVersion).complete();
                                // revert that the class has already set itself as a static member, because it considers the primary class its package.
                                // otherwise, the static initializer that inits the secondary class may not be set by _storeMember()!
                                delete joo.getQualifiedObject(this.fullClassName)[memberDeclaration.memberName];
                                memberDeclaration._static = true;
                                memberDeclaration.initSlot(this.level);
                                this._storeMember(memberDeclaration, createSecondaryClassInitializer$static(secondaryClass.publicConstructor));
                                break;
                            default:
                                for (var memberName in /*:String*/
                                member) {
                                    this._storeMember(this._createMemberDeclaration(memberDeclaration, {
                                        memberName: memberName
                                    }), member[memberName]);
                                };
                        }
                    };
                    break;
                case "object":
                    joo.SystemClassLoader.addToMetadata(metadata, item);
            }
        }
        if (!this.isInterface() && !this.native_) {
            if (!this.superClassDeclaration.constructor_) {
                throw new AS3.Error("Class " + this.fullClassName + " extends " + this.superClassDeclaration.fullClassName + " whose constructor is not defined!");
            }
            // only add "super$..." for backwards compatibility, and never if we are a JavaScriptObject:
            if (!(this.Public.prototype instanceof joo.JavaScriptObject)) {
                this.Public.prototype["super$" + this.level] = this.superClassDeclaration.constructor_;
            }
            if (!this.constructor_) {
                // no explicit constructor found
                // generate constructor invoking super() and initialize it from the "collecting" constructor:
                this._setConstructor(createSuperConstructor$static(this));
            }
        }
    }
    /*

  private static*/
    function createSecondaryClassInitializer$static(secondaryClass) /*:Function*/
    /*:Function*/
    {
        return function() /*:Function*/
        {
            // init secondary class together with primary class:
            return joo.classLoader.init(secondaryClass);
        };
    }
    /*

  //noinspection JSFieldCanBeLocalInspection
  private static*/
    var jooClasstoString$static = /*:Function*/
        null;
    function static$1() {
        jooClasstoString$static = function() /*:String*/
        {
            return "[class " + this.$class.className + "]";
        };
    }
    /*

  internal*/
    function _setConstructor(constructor_) /*:Function*/
    /*:void*/
    {
        // replay all non-private static members collected so far for new constructor_ function:
        for (var i = /*:int*/
            0; i < this.memberDeclarations.length; i++) {
            var memberDeclaration = /*:MemberDeclaration*/
                this.memberDeclarations[i];
            if (memberDeclaration.isStatic() && !memberDeclaration.isPrivate()) {
                memberDeclaration.storeMember(constructor_);
            }
        }
        constructor_['$class'] = this;
        if (this.superClassDeclaration) {
            constructor_['superclass'] = this.superClassDeclaration.Public.prototype;
        }
        // Ext Core compatibility!
        constructor_.prototype = this.Public.prototype;
        Object.defineProperty(this.Public.prototype, 'constructor', {
            value: constructor_,
            writable: true,
            configurable: true
        });
        constructor_.toString = jooClasstoString$static;
        // replace initializing constructor by the real one:
        this.package_[this.className] = this.constructor_ = constructor_;
    }
    /*

  private static*/
    function createSuperConstructor$static(classDeclaration) /*:JooClassDeclaration*/
    /*:Function*/
    {
        return function generatedConstructor$() /*:void*/
        {
            classDeclaration.constructor_["superclass"].constructor.call(this);
        };
    }
    /*

  internal*/
    function initMethod(memberDeclaration, /* : MemberDeclaration*/
    member) /* : Function*/
    /* : void*/
    {
        if (memberDeclaration.memberName == this.className && !memberDeclaration.isStatic()) {
            if (memberDeclaration.getterOrSetter) {
                throw new AS3.Error(this + ": Class name cannot be used for getter or setter: " + memberDeclaration);
            }
            if (!this.native_ && !memberDeclaration.isNative()) {
                this._setConstructor(member);
            }
        } else {
            memberDeclaration.initSlot(this.level);
            if (memberDeclaration.isNative()) {
                member = memberDeclaration.getNativeMember(this.publicConstructor);
            }
            if (memberDeclaration.isMethod()) {
                if (this.extends_ != "Object") {
                    var superMethod = /* : Function*/
                        memberDeclaration.retrieveMember(this.superClassDeclaration.Public.prototype);
                }
                var overrides = /* : Boolean*/
                    !!superMethod && superMethod !== member && superMethod !== Object['prototype'][memberDeclaration.memberName];
                if (overrides !== memberDeclaration.isOverride()) {
                    var msg = /* : String*/
                        overrides ? "Method overrides without 'override' modifier" : "Method with 'override' modifier does not override";
                    AS3.trace("[WARN]", this + ": " + msg + ": " + memberDeclaration);
                } else if (overrides) {
                    // found overriding: store super class' method as private member:
                    this._storeMember(this._createMemberDeclaration(memberDeclaration, {
                        _namespace: joo.MemberDeclaration.NAMESPACE_PRIVATE
                    }), superMethod);
                }
            }
            this._storeMember(memberDeclaration, member);
        }
    }
    /*

  internal*/
    function _createMemberDeclaration(memberDeclaration, /* : MemberDeclaration*/
    changedProperties) /* : Object*/
    /* : MemberDeclaration*/
    {
        var newMemberDeclaration = /* : MemberDeclaration*/
            memberDeclaration.clone(changedProperties);
        newMemberDeclaration.initSlot(this.level);
        return newMemberDeclaration;
    }
    /*

  internal*/
    function _storeMember(memberDeclaration, /* : MemberDeclaration*/
    value) /* : Object*/
    /* : void*/
    {
        this.memberDeclarations.push(memberDeclaration);
        this.memberDeclarationsByQualifiedName[memberDeclaration.getQualifiedName()] = memberDeclaration;
        memberDeclaration.value = value;
        var _static = /* : Boolean*/
            memberDeclaration.isStatic();
        var _private = /* : Boolean*/
            memberDeclaration.isPrivate();
        if (_static && memberDeclaration.hasInitializer()) {
            this.staticInitializers.push(memberDeclaration);
        }
        this._processMetadata(memberDeclaration);
        var target = /* : Object*/
            _static ? _private ? this.privateStatics : this.constructor_ : this.Public.prototype;
        if (target) {
            memberDeclaration.storeMember(target);
        }
    }
    // if constructor_ is not yet set, static non-private members will be added later by _setConstructor().
    /*

  internal*/
    function _processMetadata(memberDeclaration) /* : MemberDeclaration = null*/
    /*:void*/
    {
        if (arguments.length <= 0)  {
            memberDeclaration = null;
        }
        
        var metaPackage = /*:**/
            joo.getQualifiedObject("joo.meta");
        if (metaPackage) {
            var metadata = /*:Object*/
                memberDeclaration ? memberDeclaration.metadata : this.metadata;
            if (metadata) {
                for (var metaFunctionName in /*:String*/
                metadata) {
                    if (metaFunctionName in metaPackage) {
                        metaPackage[metaFunctionName](this, memberDeclaration, metadata[metaFunctionName]);
                    }
                }
            }
        }
    }
    /*

  internal override*/
    function doInit() /* : void*/
    {
        if (!this.isClass() && !this.isInterface()) {
            return;
        }
        this.superClassDeclaration.init();
        for (var j = /*:int*/
            0; j < this.interfaces.length; j++) {
            this.interfaces[j] = joo.classLoader.getRequiredClassDeclaration(this.interfaces[j]).init();
        }
        this.initMembers();
        if (this.isInterface()) {
            this.implementingClasses = [];
        } else {
            this.addToInterfaces(this.constructor_);
        }
        this.state = joo.NativeClassDeclaration.STATE_MEMBERS_INITIALIZED;
        this.fireStateEvent(JooClassDeclaration.STATE_EVENT_AFTER_INIT_MEMBERS);
        for (var i = /*:int*/
            0; i < this.staticInitializers.length; ++i) {
            var staticInitializer = /* : **/
                this.staticInitializers[i];
            if (typeof staticInitializer == "function") {
                // static statements
                staticInitializer();
            } else {
                //noinspection UnnecessaryLocalVariableJS
                var memberDeclaration = /*:MemberDeclaration*/
                    staticInitializer;
                // static variable initializer expression
                var target = /* : Object*/
                    memberDeclaration.isPrivate() ? this.privateStatics : this.constructor_;
                target[memberDeclaration.slot] = target[memberDeclaration.slot]();
            }
        }
    }
    /*

  internal*/
    function fireStateEvent(event) /*:String*/
    /*:void*/
    {
        var stateListeners = /*:Array*/
            this.stateListeners[event];
        if (stateListeners) {
            for (var i = /*:int*/
                0; i < stateListeners.length; i++) {
                stateListeners[i](this);
            }
            delete this.stateListeners[event];
        }
    }
    /*

  public*/
    function getMemberDeclaration(namespace_, /* : String*/
    memberName) /* : String*/
    /* : MemberDeclaration*/
    {
        var memberDeclaration = /*:MemberDeclaration*/
            this.memberDeclarationsByQualifiedName[namespace_ + "::" + memberName];
        return !memberDeclaration && this.superClassDeclaration && this.superClassDeclaration["getMemberDeclaration"] ? AS3.cast(JooClassDeclaration, this.superClassDeclaration).getMemberDeclaration(namespace_, memberName) : memberDeclaration;
    }
    /*

  public*/
    function getDependencies() /* : Array*/
    {
        return this.dependencies;
    }
    /*

  private static*/
    function createInitializingConstructor$static(classDeclaration) /* : JooClassDeclaration*/
    /* : Function*/
    {
        return function() /* : Object*/
        {
            arguments = Array.prototype.slice.call(arguments);
            classDeclaration.init();
            // create an uninitialized Object with the correct prototype chain:
            var instance = /*:Object*/
                new classDeclaration.Public();
            // now, apply the constructor to that Object.
            // classDeclaration.constructor_ must have been set, at least to a default constructor:
            classDeclaration.constructor_.apply(instance, arguments);
            return instance;
        };
    }
    /*

  private static*/
    function createInitializingPackageMethod$static(classDeclaration) /* : JooClassDeclaration*/
    /* : Function*/
    {
        return function() /* : **/
        {
            arguments = Array.prototype.slice.call(arguments);
            var fun = /*:Function*/
                classDeclaration.package_[classDeclaration.className] = classDeclaration.memberDeclarations();
            return fun.apply(null, arguments);
        };
    }
    /*

  private static*/
    function createInitializingPackageField$static(classDeclaration) /*:JooClassDeclaration*/
    /*:Object*/
    {
        return {
            $class: {
                init: function() /*:**/
                {
                    var value = /*:**/
                        classDeclaration.package_[classDeclaration.className] = classDeclaration.memberDeclarations();
                    return {
                        constructor_: value
                    };
                }
            }
        };
    }
    /*

  internal*/
    function createInitializingStaticMethod(methodName) /* : String*/
    /* : void*/
    {
        var this$ = this;
        this.publicConstructor[methodName] = function() /* : **/
        {
            arguments = Array.prototype.slice.call(arguments);
            this$.init();
            return this$.constructor_[methodName].apply(null, arguments);
        };
    }
    /*
}
}

============================================== Jangaroo part ==============================================*/
    return {
        extend: "joo.NativeClassDeclaration",
        package_: null,
        className: null,
        native_: false,
        extends_: "Object",
        level: -1,
        privateStatics: null,
        memberDeclarations: undefined,
        memberDeclarationsByQualifiedName: null,
        staticInitializers: null,
        publicStaticMethodNames: null,
        implementingClasses: null,
        dependencies: null,
        stateListeners: null,
        metadata: null,
        constructor: JooClassDeclaration$,
        addStateListener: addStateListener,
        removeStateListener: removeStateListener,
        isClass: isClass,
        isInterface: isInterface,
        isFunction: isFunction,
        isConst: isConst,
        isVar: isVar,
        addToInterfaces: addToInterfaces,
        addImplementingClass: addImplementingClass,
        isInstance: isInstance,
        isNamespace: isNamespace,
        isNative: isNative,
        doComplete: doComplete,
        initMembers: initMembers,
        _setConstructor: _setConstructor,
        initMethod: initMethod,
        _createMemberDeclaration: _createMemberDeclaration,
        _storeMember: _storeMember,
        _processMetadata: _processMetadata,
        doInit: doInit,
        fireStateEvent: fireStateEvent,
        getMemberDeclaration: getMemberDeclaration,
        getDependencies: getDependencies,
        createInitializingStaticMethod: createInitializingStaticMethod,
        statics: {
            STATE_EVENT_AFTER_INIT_MEMBERS: 'afterInitMembers',
            __initStatics__: function() {
                static$0();
                static$1();
            }
        },
        requires: [
            "joo.NativeClassDeclaration"
        ],
        uses: [
            "AS3.Error",
            "AS3.trace",
            "joo.JavaScriptObject",
            "joo.MemberDeclaration",
            "joo.SystemClassLoader"
        ]
    };
});

Ext.define("joo.MemberDeclaration", function(MemberDeclaration) {
    /* /*
 * Copyright 2009 CoreMedia AG
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, 
 * software distributed under the License is distributed on an "AS
 * IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either 
 * express or implied. See the License for the specific language 
 * governing permissions and limitations under the License.
 * /

// JangarooScript runtime support. Author: Frank Wienberg

package joo {

public class MemberDeclaration {

  public static const
          METHOD_TYPE_GET : String = "get",
          METHOD_TYPE_SET : String = "set",
          MEMBER_TYPE_VAR : String = "var",
          MEMBER_TYPE_CONST : String = "const",
          MEMBER_TYPE_FUNCTION : String = "function",
          MEMBER_TYPE_CLASS : String = "class",
          MEMBER_TYPE_INTERFACE : String = "interface",
          MEMBER_TYPE_NAMESPACE : String = "namespace",
          NAMESPACE_PRIVATE : String = "private",
          NAMESPACE_INTERNAL : String = "internal",
          NAMESPACE_PROTECTED : String = "protected",
          NAMESPACE_PUBLIC : String = "public",
          STATIC : String = "static",
          FINAL : String = "final",
          NATIVE : String = "native",
          OVERRIDE : String = "override",
          VIRTUAL : String = "virtual";

  private static*/
    var SUPPORTS_GETTERS_SETTERS$static = /* : Boolean*/
        false;
    /*
  private static*/
    var SUPPORTS_PROPERTIES$static = /* : Boolean*/
        false;
    /*
  private static*/
    var DEFINE_METHOD$static = /* : Object*/
        null;
    /*
  private static*/
    var LOOKUP_METHOD$static = /* : Object*/
        null;
    function static$0() {
        {
            // no static initializers in system classes, use static block:
            if ('getOwnPropertyDescriptor' in Object) {
                try {
                    // To make sure we are not in IE8, which implements this method only partially,
                    // just try getting a property of some Object and see if it fails:
                    SUPPORTS_PROPERTIES$static = Object['getOwnPropertyDescriptor']({
                        foo: 1
                    }, "foo").value === 1;
                } catch (e) /*:**/
                {}
            }
            // ignore
            SUPPORTS_GETTERS_SETTERS$static = "__defineGetter__" in Object['prototype'];
            DEFINE_METHOD$static = {
                "get": "__defineGetter__",
                "set": "__defineSetter__"
            };
            LOOKUP_METHOD$static = {
                "get": "__lookupGetter__",
                "set": "__lookupSetter__"
            };
        }
    }
    /*

  public static*/
    function create$static(memberDeclarationStr) /* : String*/
    /* : MemberDeclaration*/
    {
        var tokens = /* : Array*/
            memberDeclarationStr.split(/\s+/);
        // ignore imports:
        return tokens[0] == "import" ? null : new MemberDeclaration(tokens);
    }
    /*

  internal var
          _namespace : String =*/
    function _namespace_() {
        this._namespace = (MemberDeclaration.NAMESPACE_INTERNAL);
    }
    /*,
          _static : Boolean = false,
          _final : Boolean = false,
          _native : Boolean = false,
          _override : Boolean = false,
          _cloneFactory : Function;
  public var
          memberType : String,
          getterOrSetter : String,
          memberName : String,
          slot : String,
          value : *;
  /**
   * The metadata (annotations) associated with this member.
   * /
  public var metadata : Object =*/
    function metadata_() {
        this.metadata = ({});
    }
    /*;

  public*/
    function MemberDeclaration$(tokens) /* : Array*/
    {
        _namespace_.call(this);
        metadata_.call(this);
        for (var j = /*:int*/
            0; j < tokens.length; ++j) {
            var token = /* : String*/
                tokens[j];
            if (!this.memberType) {
                switch (token) {
                    case MemberDeclaration.STATIC:
                    case MemberDeclaration.FINAL:
                    case MemberDeclaration.NATIVE:
                    case MemberDeclaration.OVERRIDE:
                        this["_" + token] = true;
                        break;
                    case MemberDeclaration.MEMBER_TYPE_VAR:
                    case MemberDeclaration.MEMBER_TYPE_CONST:
                    case MemberDeclaration.MEMBER_TYPE_FUNCTION:
                    case MemberDeclaration.MEMBER_TYPE_CLASS:
                        this.memberType = token;
                        break;
                    case MemberDeclaration.VIRTUAL:
                        break;
                    // ignore, but do not consider a namespace
                    default:
                        // "private", "public", "protected", "internal" or a custom namespace:
                        this._namespace = token;
                }
            } else {
                if (this.isMethod() && LOOKUP_METHOD$static[this.memberName]) {
                    this.getterOrSetter = this.memberName;
                }
                // detected getter or setter
                this.memberName = token;
                // token following the member type is the member name
                if (this.memberType === MemberDeclaration.MEMBER_TYPE_CLASS) {
                    break;
                }
            }
        }
        // let classLoader.prepare parse the rest!
        if (!this.memberType) {
            throw new AS3.Error("Missing member type in declaration '" + tokens.join(" ") + "'.");
        }
    }
    /*

  public*/
    function getQualifiedName() /* : String*/
    {
        return this._namespace + "::" + this.memberName;
    }
    /*

  public*/
    function isPrivate() /* : Boolean*/
    {
        return this._namespace == MemberDeclaration.NAMESPACE_PRIVATE;
    }
    /*

  public*/
    function isStatic() /* : Boolean*/
    {
        return this._static;
    }
    /*

  public*/
    function isFinal() /* : Boolean*/
    {
        return this._final;
    }
    /*

  public*/
    function isNative() /* : Boolean*/
    {
        return this._native;
    }
    /*

  public*/
    function isOverride() /* : Boolean*/
    {
        return this._override;
    }
    /*

  public*/
    function isMethod() /* : Boolean*/
    {
        return this.memberType == MemberDeclaration.MEMBER_TYPE_FUNCTION;
    }
    /*

  internal*/
    function initSlot(level) /* : int*/
    /* : void*/
    {
        this.slot = this.isPrivate() && !this.isStatic() ? this.memberName + "$" + level : this.memberName;
    }
    /*

  // public function retrieveMember(source : Object) : Function
  /* not needed if we take reflection seriously!
   retrieveMember: function joo$MemberDeclaration$getMember(source) {
   return this.getterOrSetter==METHOD_TYPE_GET ? source.__lookupGetter__(this.memberName)
   : this.getterOrSetter==METHOD_TYPE_SET ? source.__lookupSetter__(this.memberName)
   : source[this.memberName];
   },* /

  public*/
    function getNativeMember(publicConstructor) /* : Function*/
    /* : **/
    {
        var target = /* : **/
            this.isStatic() ? publicConstructor : publicConstructor.prototype;
        if (this.memberType == MemberDeclaration.MEMBER_TYPE_FUNCTION && this.getterOrSetter) {
            // native variables are only declared as getter/setter functions, never implemented as such:
            this.memberType = MemberDeclaration.MEMBER_TYPE_VAR;
            this.getterOrSetter = null;
        }
        try {
            var member = /* : **/
                target[this.memberName];
        } catch (e) /* : **/
        {}
        // ignore Firefox' native member access exceptions.
        if (typeof member != "function") {
            var memberObject = /* : Object*/
                {};
            memberObject[this.memberName] = member;
            member = memberObject;
        }
        return member;
    }
    /*

  public*/
    function hasOwnMember(target) /* : Object*/
    /* : Boolean*/
    {
        // fast path:
        if (!this.getterOrSetter && "hasOwnProperty" in target) {
            return target.hasOwnProperty(this.slot);
        }
        var value = /* : **/
            this.retrieveMember(target);
        if (value !== undefined && target) {
            // is it really target's own member? Retrieve super's value:
            var superTarget = /* : Object*/
                Object.getPrototypeOf(target);
            var superValue = /* : **/
                this.retrieveMember(superTarget);
            if (value !== superValue) {
                return true;
            }
        }
        return false;
    }
    /*

  public*/
    function retrieveMember(target) /* : Object*/
    /* : **/
    {
        if (!target) {
            return undefined;
        }
        var slot = /* : String*/
            this.slot;
        if (this.getterOrSetter) {
            if (SUPPORTS_PROPERTIES$static) {
                var propertyDescriptor = /*:Object*/
                    this._lookupPropertyDescriptor(target);
                return propertyDescriptor ? propertyDescriptor[this.getterOrSetter] : undefined;
            } else if (SUPPORTS_GETTERS_SETTERS$static) {
                return target[LOOKUP_METHOD$static[this.getterOrSetter]](slot);
            } else {
                slot = slot + "$" + this.getterOrSetter;
            }
        }
        try {
            return target[slot];
        } catch (e) /*:**/
        {
            // some IE oddities may throw error instead of returning undefined
            return undefined;
        }
    }
    /*

  internal*/
    function _lookupPropertyDescriptor(target) /*:Object*/
    /*:Object*/
    {
        var slot = /*:String*/
            this.slot;
        do {
            var propertyDescriptor = Object['getOwnPropertyDescriptor'](target, slot);
            if (propertyDescriptor) {
                return propertyDescriptor;
            }
            var oldTarget = target;
            target = Object.getPrototypeOf(target);
        } while (/*:Object*/
        /*:Object*/
        target && target !== oldTarget);
        return undefined;
    }
    /*

  public*/
    function storeMember(target) /* : Object*/
    /* : void*/
    {
        // store only if not native and not already present:
        if (!this.isNative() && !this.hasOwnMember(target)) {
            var slot = /* : String*/
                this.slot;
            if (this.getterOrSetter) {
                if (SUPPORTS_PROPERTIES$static) {
                    // we have to redefine the property as a whole, so we look up the existing definition first:
                    var propertyDescriptor = /*:Object*/
                        this._lookupPropertyDescriptor(target) || {
                            configurable: true,
                            enumerable: true
                        };
                    propertyDescriptor[this.getterOrSetter] = this.value;
                    Object['defineProperty'](target, slot, propertyDescriptor);
                    return;
                } else if (SUPPORTS_GETTERS_SETTERS$static) {
                    // defining a getter or setter disables the counterpart setter/getter from the prototype,
                    // so copy that setter/getter before, if "target" does not already define it:
                    var oppositeMethodType = /*:**/
                        this.getterOrSetter == MemberDeclaration.METHOD_TYPE_GET ? MemberDeclaration.METHOD_TYPE_SET : MemberDeclaration.METHOD_TYPE_GET;
                    var counterpart = /* : Function*/
                        target[LOOKUP_METHOD$static[oppositeMethodType]](slot);
                    // if counterpart is defined, check that it is not overridden (differs from prototype's counterpart):
                    if (counterpart && counterpart === Object.getPrototypeOf(target)[LOOKUP_METHOD$static[oppositeMethodType]](slot)) {
                        // set the counterpart directly on target. This may be redundant, but we cannot find out.
                        target[DEFINE_METHOD$static[oppositeMethodType]](slot, counterpart);
                    }
                    target[DEFINE_METHOD$static[this.getterOrSetter]](slot, this.value);
                    return;
                } else {
                    slot = slot + "$" + this.getterOrSetter;
                }
            }
            target[slot] = this.value;
        }
    }
    /*

  public*/
    function hasInitializer() /* : Boolean*/
    {
        return this.memberType === MemberDeclaration.MEMBER_TYPE_CLASS || this.memberType !== MemberDeclaration.MEMBER_TYPE_FUNCTION && typeof this.value === 'function' && this.value.constructor !== RegExp;
    }
    /*

  public*/
    function _getCloneFactory() /* : Function*/
    {
        if (!this._cloneFactory) {
            this._cloneFactory = function() /* : void*/
            {};
            this._cloneFactory.prototype = this;
        }
        return this._cloneFactory;
    }
    /*

  public*/
    function clone(changedProperties) /* : Object*/
    /* : MemberDeclaration*/
    {
        var CloneFactory = /* : Function*/
            this._getCloneFactory();
        var clone = /* : MemberDeclaration*/
            AS3.cast(MemberDeclaration, new CloneFactory());
        for (var m in /*:String*/
        changedProperties) {
            clone[m] = changedProperties[m];
        }
        return clone;
    }
    /*

  public*/
    function toString() /* : String*/
    {
        var sb = /* : Array*/
            [
                this._namespace
            ];
        if (this._static) {
            sb.push(MemberDeclaration.STATIC);
        }
        if (this._override) {
            sb.push(MemberDeclaration.OVERRIDE);
        }
        sb.push(this.memberType);
        if (this.getterOrSetter) {
            sb.push(this.getterOrSetter);
        }
        sb.push(this.memberName);
        return sb.join(" ");
    }
    /*

}
}

============================================== Jangaroo part ==============================================*/
    return {
        _static: false,
        _final: false,
        _native: false,
        _override: false,
        _cloneFactory: null,
        memberType: null,
        getterOrSetter: null,
        memberName: null,
        slot: null,
        value: undefined,
        constructor: MemberDeclaration$,
        getQualifiedName: getQualifiedName,
        isPrivate: isPrivate,
        isStatic: isStatic,
        isFinal: isFinal,
        isNative: isNative,
        isOverride: isOverride,
        isMethod: isMethod,
        initSlot: initSlot,
        getNativeMember: getNativeMember,
        hasOwnMember: hasOwnMember,
        retrieveMember: retrieveMember,
        _lookupPropertyDescriptor: _lookupPropertyDescriptor,
        storeMember: storeMember,
        hasInitializer: hasInitializer,
        _getCloneFactory: _getCloneFactory,
        clone: clone,
        toString: toString,
        statics: {
            METHOD_TYPE_GET: "get",
            METHOD_TYPE_SET: "set",
            MEMBER_TYPE_VAR: "var",
            MEMBER_TYPE_CONST: "const",
            MEMBER_TYPE_FUNCTION: "function",
            MEMBER_TYPE_CLASS: "class",
            MEMBER_TYPE_INTERFACE: "interface",
            MEMBER_TYPE_NAMESPACE: "namespace",
            NAMESPACE_PRIVATE: "private",
            NAMESPACE_INTERNAL: "internal",
            NAMESPACE_PROTECTED: "protected",
            NAMESPACE_PUBLIC: "public",
            STATIC: "static",
            FINAL: "final",
            NATIVE: "native",
            OVERRIDE: "override",
            VIRTUAL: "virtual",
            create: create$static,
            __initStatics__: function() {
                static$0();
            }
        },
        uses: [
            "AS3.Error"
        ]
    };
});

